from __future__ import print_function

import os
import ast
import json
import time
import argparse

from biblio import Biblio
from biblio.util import walk_path
from biblio.walker import Walker
from biblio.outputs import OUTPUTS
from biblio.parsers import PARSERS
from biblio.config import load_config, match_path
from biblio.util import walk_path

parser = argparse.ArgumentParser()
parser.add_argument('config', help='Path to configuration file')
parser.add_argument('--debug', action='store_true')


def main():
    args = parser.parse_args()
    config = load_config(args.config)
    path = os.path.dirname(args.config)
    b = Biblio(path, config, debug=args.debug)
    b.run()


    '''
    # Determine and build output
    otyp = OUTPUTS.get(config['output'].pop('type'))
    if not otyp:
        print('ERROR: invalid output specified')
        return
    output = otyp(config['output'])

    # Determine and build parser
    ptyp = PARSERS.get(config.get('parser', 'numpy'))
    if not ptyp:
        print('ERROR: invalid parser specified')
        return
    doc_parser = ptyp()

    # Determine path to scan
    path = os.path.dirname(args.config)
    if 'path' in config:
        path = os.path.join(path, config['path'])

    modules = []

    print('[WALK] --- START ---')
    start = time.time()
    for file_path in walk_path(path):
        module_path = file_path.replace(os.path.commonprefix([path, file_path]) + '/', '')
        module_name = module_path[:-3].replace('/', '.')
        if not match_path(config, module_path):
            continue

        with open(file_path, 'r') as f:
            print('[WALK] {}'.format(module_name))
            module = Walker(doc_parser).visit(ast.parse(f.read()))
            modules.append((module_name, module))
    print('[WALK] --- END (%ss) ---' % (time.time() - start))

    print('[OUTPUT] --- START ---')
    start = time.time()
    output.begin(modules)
    for module_name, module in modules:
        print('[OUTPUT] {}'.format(module_name))
        output.output_module(module_name, module)
    output.complete(modules)
    print('[OUTPUT] --- END (%ss) ---' % (time.time() - start))

    if args.debug:
        with open(args.debug, 'w') as f:
            json.dump(modules, f, indent=2, sort_keys=True)
    '''

if __name__ == '__main__':
    main()



import os
import fnmatch


def walk_path(path, rule):
    for root, dirs, files in os.walk(path):
        if '.git' in root:
            continue

        for file_name in files:
            path = os.path.join(root, file_name)[2:]
            if fnmatch.fnmatch(path, rule):
                yield path



import os
import sys

from jinja2 import Environment, FileSystemLoader

from ..base import BaseOutput


class MarkdownOutput(BaseOutput):
    def __init__(self, *args, **kwargs):
        super(MarkdownOutput, self).__init__(*args, **kwargs)
        default_path = os.path.join(os.path.dirname(os.path.realpath(__file__)), 'templates')
        self.env = Environment(
            loader=FileSystemLoader(self.config.get('template_path', default_path)))
        self.env.filters['clean'] = lambda s: s.replace('\n', ' ').replace('|', '&#124;').replace('`', '&#96;')

    def process_docstring(self, docstring):
        return docstring

    def output_module(self, name, module):
        with open(self.module_path(name, 'md'), 'w') as f:
            template = self.env.get_template('module.md')
            data = template.render(module=module, name=name)

            if (sys.version_info > (3, 0)):
                f.write(data)
            else:
                f.write(data.encode('utf-8'))



import os
import ast
import six
import logging

from biblio.util import walk_path
from biblio.walker import Walker
from biblio.outputs import OUTPUTS
from biblio.parsers import PARSERS


logging.basicConfig(
    format='[%(levelname)s] %(asctime)s - %(name)s:%(lineno)d - %(message)s'
)


class Biblio(object):
    def __init__(self, path, config, debug=False):
        self.path = path
        self.config = config
        self.log = logging.getLogger(__name__)

        if debug:
            logging.getLogger().setLevel(logging.DEBUG)
        else:
            logging.getLogger().setLevel(logging.INFO)

    def run(self):
        modules = self._walk()
        for output in self.config['outputs']:
            self._output(output, modules)
        pass

    def _output(self, output, modules):
        self.log.info('Running output %s on %s modules', output['type'], len(modules))

        output_type = OUTPUTS.get(output['type'])
        if not output_type:
            raise Exception('Invalid output `{}` requested'.format(output['type']))
        output = output_type(output)

        output.begin(modules)
        for name, module in modules:
            self._output_module(name, module, output)
        output.complete(modules)

    def _output_module(self, name, module, output):
        self.log.info('outputting module %s', name)
        output.output_module(name, module)

    def _walk(self):
        self.log.info('Walking %s rules', len(self.config['rules']))
        return reduce(sum, [list(self._walk_rule(*i)) for i in six.iteritems(self.config['rules'])])

    def _walk_rule(self, rule, options):
        self.log.info('Walking rule %s', rule)

        curdir = os.getcwd()
        os.chdir(self.path)

        parser_type = PARSERS.get(options.get('parser', 'numpy'))
        if not parser_type:
            raise Exception('Invalid parser `{}` requested'.format(options.get('parser')))
        doc_parser = parser_type()

        for file_path in walk_path('.', rule):
            yield self._process_file(file_path, doc_parser)

        os.chdir(curdir)

    def _process_file(self, file_path, doc_parser):
        self.log.debug('processing file %s', file_path)
        module_name = os.path.splitext(file_path)[0].replace('/', '.')

        if module_name.endswith('__init__'):
            module_name = module_name[:-9]

        with open(file_path, 'r') as f:
            return module_name, Walker(doc_parser).visit(ast.parse(f.read()))



import os


class BaseOutput(object):
    def __init__(self, config):
        self.config = config
        self.index = {}

        if 'path' in config:
            self.path = config['path']

        if not os.path.exists(self.path):
            os.mkdir(self.path)

    def module_path(self, module, ext):
        return os.path.join(self.path, module.replace('.', '_') + '.' + ext)

    def output_module(self, name, module):
        return None

    def begin(self, modules):
        pass

    def complete(self, modules):
        pass



import re

try:
    from itertools import izip
except ImportError:
    izip = zip

from biblio.parsers.base import BaseParser


SECTION_HEADER_RE = re.compile(r'(\w+)\n[-]+')
SECTION_CONTENT_RE = re.compile(r'^(\S+)(?:$|[ ]+:[ ]+?(.+))$', re.M)

SECTIONS = [
    'args',
    'arguments',
    'attributes',
    'example',
    'examples',
    'kwargs',
    'methods',
    'notes',
    'note',
    'parameters',
    'params',
    'return',
    'returns',
    'raises',
    'references',
    'todo',
    'see also',
    'warning',
    'warnings',
    'warns',
    'yield',
    'yields',
]


def nwise(iterable, n):
    return izip(*[
        iterable[i::n] for i in range(n)
    ])


class NumPyParser(BaseParser):
    def parse(self, docstring):
        result = {
            'raw': docstring,
            'sections': {}
        }

        if not docstring:
            return result

        for name, section in self.get_sections(docstring):
            if section in result:
                raise Exception('More than one section for {} defined'.format(section))

            parts = []
            for a, b, c in self.get_section_content(section):
                parts.append({
                    'name': a,
                    'type': b,
                    'content': c,
                })

            result['sections'][name.lower()] = parts

        return result

    def get_sections(self, docstring):
        parts = [i.strip() for i in SECTION_HEADER_RE.split(docstring)]
        if not len(parts):
            raise StopIteration

        # If the first part isn't in the sections, we can safely assume its the pre-doc comment
        if parts[0].lower() not in SECTIONS:
            yield 'header', parts.pop(0)

        for header, section in nwise(parts, 2):
            yield header, section

    def get_section_content(self, section):
        parts = SECTION_CONTENT_RE.split(section)

        if parts[0]:
            yield None, None, parts[0]

        for item in nwise(parts[1:], 3):
            yield item



import yaml
import fnmatch


def load_config(path):
    with open(path, 'r') as f:
        return yaml.load(f)


def match_path(config, path):
    for rule in config['rules']:
        if not fnmatch.fnmatch(path, rule):
            return False

    return True



from __future__ import print_function

try:
    from itertools import imap, ifilter
except:
    imap = map
    ifilter = filter

import ast


class Walker(ast.NodeVisitor):
    filt = None

    def __init__(self, parser):
        self.parser = parser

    def get_docstring(self, node):
        return self.parser.parse(ast.get_docstring(node))

    def generic_visit(self, node):
        if not hasattr(self, 'visit_{}'.format(node.__class__.__name__)):
            print('WARNING: unvisited class: {}'.format(node))
            print(node.__dict__)
        return super(Walker, self).generic_visit(node)

    def visit_Module(self, node):
        obj = {
            'type': 'module',
            'docstring': self.get_docstring(node),
            'classes': [],
            'functions': [],
            'variables': [],
            'imports': [],
        }

        for node in imap(self.visit, node.body):
            if node['type'] == 'function':
                obj['functions'].append(node)
            elif node['type'] == 'class':
                obj['classes'].append(node)
            elif node['type'] == 'assign':
                obj['variables'].append(node)
            elif node['type'] in ('import', 'import_from'):
                obj['imports'].append(node)
            elif node['type'] in ('expr', ):
                continue

        return obj

    def visit_ClassDef(self, node):
        obj = {
            'type': 'class',
            'name': node.name,
            'docstring': self.get_docstring(node),
            'bases': list(ifilter(lambda k: k.get('name') != 'object', [
                {'name': i.id} if isinstance(i, ast.Name) else self.visit(i) for i in node.bases
            ])),
            'attributes': [],
            'functions': [],
        }

        for node in imap(self.visit, node.body):
            if node['type'] == 'function':
                obj['functions'].append(node)
            elif node['type'] == 'assign':
                obj['attributes'].append(node)

        return obj

    def visit_FunctionDef(self, node):
        return {
            'type': 'function',
            'docstring': self.get_docstring(node),
            'decorators': [],
            'name': node.name,
            'args': self.visit(node.args)
        }

    def visit_Assign(self, node):
        return {
            'type': 'assign',
            'targets': list(imap(self.visit, node.targets)),
            'value': self.visit(node.value)
        }

    def visit_Import(self, node):
        return {
            'type': 'import',
            'names': list(imap(self.visit, node.names))
        }

    def visit_ImportFrom(self, node):
        return {
            'type': 'import_from',
            'module': node.module,
            'names': list(imap(self.visit, node.names))
        }

    def visit_Name(self, node):
        return node.id

    def visit_Attribute(self, node):
        return {
            'type': 'attribute',
            'name': node.attr,
            'value': self.visit(node.value)
        }

    def visit_Load(self, node):
        return {
            'type': 'load',
        }

    def visit_Call(self, node):
        return {
            'type': 'call',
            'name': self.visit(node.func),
            # TODO: py3
            # 'starargs': node.starargs,
            # 'kwargs': node.kwargs,
            'keywords': list(imap(self.visit, node.keywords)),
            'args': list(imap(self.visit, node.args)),
        }

    def visit_Str(self, node):
        return node.s

    def visit_Num(self, node):
        return node.n

    def visit_Dict(self, node):
        return {
            'type': 'dict',
            'keys': list(imap(self.visit, node.keys)),
            'values': list(imap(self.visit, node.values)),
        }

    def visit_Set(self, node):
        return {
            'type': 'set',
            'elts': list(imap(self.visit, node.elts)),
        }

    def visit_List(self, node):
        return {
            'type': 'list',
            'elts': list(imap(self.visit, node.elts)),
        }

    def visit_Tuple(self, node):
        return {
            'type': 'tuple',
            'elts': list(imap(self.visit, node.elts)),
        }

    def visit_Lambda(self, node):
        return {
            'type': 'lambda',
            'args': self.visit(node.args),
        }

    def visit_NameConstant(self, node):
        return node.value

    def visit_keyword(self, node):
        return {
            'type': 'keyword',
            'arg': node.arg,
            'value': self.visit(node.value),
        }

    def visit_alias(self, node):
        return node.asname or node.name

    def visit_arguments(self, node):
        return {
            'args': list(imap(self.visit, node.args)),
            'defaults': list(imap(self.visit, node.defaults)),
            'kwargs': node.kwarg,
            'vargs': node.vararg,
        }

    def visit_arg(self, node):
        return node.arg

    def visit_Param(self, node):
        return {
            'type': 'param',
        }

    def visit_If(self, node):
        return {
            'type': 'if',
        }

    def visit_TryExcept(self, node):
        return {
            'type': 'try_except',
        }

    def visit_Expr(self, node):
        return {
            'type': 'expr',
        }

    def visit_Pass(self, node):
        return {
            'type': 'pass',
        }

    def visit_Add(self, node):
        return {
            'type': 'add',
        }

    def visit_BinOp(self, node):
        return {
            'type': 'bin_op',
        }

    def visit_Store(self, node):
        return {
            'type': 'store',
        }

    def visit_Subscript(self, node):
        return {
            'type': 'subscript',
        }

    def visit_Index(self, node):
        return {
            'type': 'index',
        }

    def visit_ExceptHandler(self, node):
        return {
            'type': 'except_handler',
        }

    def visit_Try(self, node):
        return {
            'type': 'try',
        }



from __future__ import print_function

import os
import re
import sys
import shutil

from jinja2 import Environment, FileSystemLoader

from ..base import BaseOutput


REFERENCE_RE = re.compile(r'`(\w[^`]+)`')


class HTMLOutput(BaseOutput):
    def __init__(self, *args, **kwargs):
        super(HTMLOutput, self).__init__(*args, **kwargs)
        default_path = os.path.join(os.path.dirname(os.path.realpath(__file__)), 'templates')
        self.env = Environment(
            loader=FileSystemLoader(self.config.get('template_path', default_path)))
        self.env.filters['heading'] = self.heading
        self.env.filters['pdoc'] = self.process_docstring

        self.classes = {}
        self.classes_full = {}

    @staticmethod
    def heading(level):
        return 'h{}'.format(level)

    def process_docstring(self, docstring):
        def _f(match):
            match = match.group(1)
            if match in self.classes:
                obj = self.classes[match]
            elif match in self.classes_full:
                obj = self.classes_full[match]
            else:
                return '<code>{}</code>'.format(match)
            return '<a href="/{}.html#{}"><code>{}</code></a>'.format(
                obj[0].replace('.', '_'),
                obj[1]['name'],
                match
            )

        return REFERENCE_RE.sub(_f, docstring)

    def output_module(self, name, module):
        with open(self.module_path(name, 'html'), 'w') as f:
            template = self.env.get_template('module.html')
            data = template.render(module=module, name=name, config=self.config, modules=self.modules)

            if (sys.version_info > (3, 0)):
                f.write(data)
            else:
                f.write(data.encode('utf-8'))

    def process_module(self, name, module):
        for cls in module['classes']:
            if cls['name'] in self.classes:
                del self.classes[cls['name']]
            else:
                self.classes[cls['name']] = (name, cls)

            full = name + '.' + cls['name']
            if full in self.classes_full:
                del self.classes_full[full]
            else:
                self.classes_full[full] = (name, cls)

    def begin(self, modules):
        self.modules = modules

        for name, module in modules:
            self.process_module(name, module)

    def complete(self, modules):
        with open(os.path.join(self.path, 'index.html'), 'w') as f:
            f.write(self.env.get_template('index.html').render(modules=sorted(modules, key=lambda i: i[0]), config=self.config))

        static_path = os.path.join(os.path.dirname(os.path.realpath(__file__)), 'static') + '/'
        shutil.rmtree(os.path.join(self.path, 'static'))
        shutil.copytree(static_path, os.path.join(self.path, 'static'))



from setuptools import setup, find_packages


def run_tests():
    import unittest
    test_loader = unittest.TestLoader()
    test_suite = test_loader.discover('tests', pattern='test_*.py')
    return test_suite


with open('requirements.txt') as f:
    requirements = f.readlines()

with open('README.md') as f:
    readme = f.read()

setup(
    name='biblio',
    author='b1nzy',
    url='https://github.com/b1naryth1ef/biblio',
    version='0.0.1',
    packages=find_packages(),
    license='MIT',
    description='an extremely fast and simple documentation generator',
    long_description=readme,
    include_package_data=True,
    install_requires=requirements,
    test_suite='setup.run_tests',
    classifiers=[
        'Development Status :: 4 - Beta',
        'License :: OSI Approved :: MIT License',
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Topic :: Internet',
        'Topic :: Software Development :: Libraries',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: Utilities',
    ])



from __future__ import absolute_import

import json
from .base import BaseOutput


class JSONOutput(BaseOutput):
    def output_module(self, name, module):
        with open(self.module_path(name, 'json'), 'w') as f:
            json.dump(module, f, indent=2, sort_keys=True)
